==> apptainer-1.1.8.tar.gz: apptainer-1.1.8/internal/pkg/util/gpu/rocm.go <==
// Copyright (c) Contributors to the Apptainer project, established as
//   Apptainer a Series of LF Projects LLC.
//   For website terms of use, trademark policy, privacy policy and other
//   project policies see https://lfprojects.org/policies
// Copyright (c) 2018-2021, Sylabs Inc. All rights reserved.
// This software is licensed under a 3-clause BSD license. Please consult the
// LICENSE.md file distributed with the sources of this project regarding your
// rights to use or distribute this software.

package gpu

import (
	"fmt"
	"path/filepath"

	"github.com/apptainer/apptainer/internal/pkg/util/paths"
)

// RocmPaths returns a list of rocm libraries/binaries that should be
// mounted into the container in order to use AMD GPUs
func RocmPaths(configFilePath string) ([]string, []string, error) {
	rocmFiles, err := gpuliblist(configFilePath)
	if err != nil {
		return nil, nil, fmt.Errorf("could not read %s: %v", filepath.Base(configFilePath), err)
	}

	return paths.Resolve(rocmFiles)
}

// RocmDevices return list of all non-GPU rocm devices present on host. If withGPU
// is true all GPUs are included in the resulting list as well.
func RocmDevices(withGPU bool) ([]string, error) {
	// Must bind in all GPU DRI devices
	rocmGlob := "/dev/dri/card*"
	if !withGPU {
		rocmGlob = "/dev/dri/card[^0-9]*"
	}
	devs, err := filepath.Glob(rocmGlob)
	if err != nil {
		return nil, fmt.Errorf("could not list rocm devices: %v", err)
	}
	// render devices needed as well
	rocmGlob2 := "/dev/dri/render*"
	devs2, err2 := filepath.Glob(rocmGlob2)
	if err2 == nil && devs2 != nil {
		devs = append(devs, devs2...)
	}
	// /dev/kfd is also required
	devs = append(devs, "/dev/kfd")
	return devs, nil
}

==> apptainer-1.4.1.tar.gz: apptainer-1.4.1/internal/pkg/util/gpu/rocm.go <==
// Copyright (c) Contributors to the Apptainer project, established as
//   Apptainer a Series of LF Projects LLC.
//   For website terms of use, trademark policy, privacy policy and other
//   project policies see https://lfprojects.org/policies
// Copyright (c) 2018-2021, Sylabs Inc. All rights reserved.
// This software is licensed under a 3-clause BSD license. Please consult the
// LICENSE.md file distributed with the sources of this project regarding your
// rights to use or distribute this software.

package gpu

import (
	"fmt"
	"os"
	"path/filepath"

	"github.com/apptainer/apptainer/internal/pkg/util/paths"
)

// RocmPaths returns a list of rocm libraries/binaries that should be
// mounted into the container in order to use AMD GPUs
func RocmPaths(configFilePath string) ([]string, []string, error) {
	rocmFiles, err := gpuliblist(configFilePath)
	if err != nil {
		return nil, nil, fmt.Errorf("could not read %s: %v", filepath.Base(configFilePath), err)
	}
	// return nil slices to signal that the input was empty
	if len(rocmFiles) == 0 {
		return nil, nil, nil
	}

	libs, bins, _, err := paths.Resolve(rocmFiles)
	return libs, bins, err
}

// RocmDevices returns a list of /dev entries required for ROCm functionality.
func RocmDevices() ([]string, error) {
	// Use same paths as ROCm Docker container documentation.
	// Must bind in all GPU DRI devices, and /dev/kfd device.
	devs := []string{}
	if _, err := os.Stat("/dev/dri"); err == nil {
		devs = append(devs, "/dev/dri")
	}
	if _, err := os.Stat("/dev/kfd"); err == nil {
		devs = append(devs, "/dev/kfd")
	}
	return devs, nil
}
